// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
// Qt-Security score:significant reason:default


//
//  W A R N I N G
//  -------------
//
// This file is not part of the QtGraphs API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
#ifndef QTGRAPHS_QGRAPHSWIDGETGLOBAL_H
#define QTGRAPHS_QGRAPHSWIDGETGLOBAL_H
#include <QtCore/qglobal.h>
#include <QtGraphsWidgets/qtgraphswidgetsexports.h>

#endif // QTGRAPHS_QGRAPHSWIDGETGLOBAL_H
