/* This file was generated by syncqt. */
#ifndef QT_QTGRAPHSWIDGETS_VERSION_H
#define QT_QTGRAPHSWIDGETS_VERSION_H

#define QTGRAPHSWIDGETS_VERSION_STR "6.11.0"

#define QTGRAPHSWIDGETS_VERSION 0x0601100

#endif // QT_QTGRAPHSWIDGETS_VERSION_H
